"""This module is for the common settings of each
Sphinx's conf.py module.
"""

import warnings
from typing import Dict
from typing import List

from sphinx.application import Sphinx

PROJECT: str = "apysc"
AUTHOR: str = "simonritchie"
EXTENSIONS: List[str] = [
    "myst_parser",
]
TEMPLATES_PATH: List[str] = ["_templates"]
SOURCE_SUFFIX: Dict[str, str] = {
    ".rst": "restructuredtext",
    ".md": "markdown",
}
HTML_THEME: str = "groundwork"
HTML_STATIC_PATH: List[str] = ["../_static"]
HTML_CSS_FILES: List[str] = [
    "base.css",
    "codeblock.css",
    "iframe.css",
]
HTML_LOGO: str = "../_static/logo_for_document.png"
HTML_COPY_SOURCE: bool = False


def setup(*, sphinx: Sphinx) -> None:
    """
    The Sphinx calls when it starts building.

    Parameters
    ----------
    sphinx : Sphinx
        The Sphinx instance.
    """
    warnings.filterwarnings(
        action="ignore",
        category=UserWarning,
        message=r".*Container node skipped.*",
    )

    sphinx.add_js_file(filename="jquery-3.7.1.slim.min.js")
    sphinx.add_js_file(filename="common_func.js")
    sphinx.add_js_file(filename="add_navigation_to_footer.js")
    sphinx.add_js_file(filename="keyword_link_mapping.js")



"""This module is for the translation mapping data of the
following document:

Document file: branch_instruction_variables_reverting_setting.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Each branch instruction class's scope variable reverting setting": "# 各分岐制御のクラスのスコープ内の変数値の復元設定",  # noqa
    ##################################################
    "This page explains each branch instruction class (like the `If`\\, `Elif`\\, and `Else`) scope variables reverting setting.": "このページでは`If`や`Elif`、`Else`などの分岐制御の各クラスのスコープ内の変数の復元設定について説明します。",  # noqa
    ##################################################
    "## These interfaces execute with statement code": "## 各インターフェイスのwithステートメント内のコードの実行について",  # noqa
    ##################################################
    "These interfaces execute the code in each branch instruction regardless of the condition, and variables are updated.": "これらのインターフェイスでは条件に罹らわず各分岐箇所の（JavaScriptのコード出力のために）コードが実行され、Python上での変数の値が更新されます。",  # noqa
    ##################################################
    "For example, the following code of the condition is `False`\\, but the value of int is 20 on the Python:": "例えば以下のコード例では条件は`False`となっていますがPython上の値は20に更新されます:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\nassert int_1 == 20\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\nassert int_1 == 20\n```",  # noqa
    ##################################################
    "This condition is skipped in JavaScript (converted code) since the condition is not satisfied.": "この部分はJavaScriptへ変換されたコード上では条件を満たさないため実行されません。",  # noqa
    ##################################################
    "## Scope variables reverting setting": "## スコープ内の変数の復元設定",
    ##################################################
    "The `If`, `Elif`, and `Else` classes have the arguments of the `locals_` and `globals_` (basically set the `locals()` and `globals()` built-in functions return value). If these arguments are specified, the scope variables are reverted when ended each scope (e.g., `If` scope).": "`If`や`Elif`、`Else`などのクラスは`locals_`と`globals_`の引数の省略可能なオプションを持っています（基本的に設定する場合にはビルトインの`locals()`関数と`globals()`関数の値を設定します）。これらの引数へ値が設定された場合にはスコープ内の各変数が`If`クラスなどのそれぞれのスコープが終了した時点でスコープ前の段階に復元されます。",  # noqa
    ##################################################
    "This interface is occasionally helpful when you don't want to update the variables in each branch instruction scope.": "このインターフェイスのオプションは各分岐の箇所でPythonの変数を更新したく無い場合などに役に立つケースがあります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition, locals_=locals(), globals_=globals()):\n    int_1 += 10\nassert int_1 == 10\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition, locals_=locals(), globals_=globals()):\n    int_1 += 10\nassert int_1 == 10\n```",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [If class](if.md)": "- [If クラス](jp_if.md)",
    ##################################################
    "- [Elif class](elif.md)": "- [Elif クラス](jp_elif.md)",
    ##################################################
    "- [Else class](else.md)": "- [Else クラス](jp_else.md)",
}



"""This module is for the translation mapping data of the
following document:

Document file: fps.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# FPS enum": "# FPS の enum",
    ##################################################
    "This page explains the `FPS` enum class.": "このページでは`FPS`のenumのクラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `FPS` enum class is the definition of each FPS (frames per second). The timer uses this enum to determine the timer tick interval.": "`FPS`のenumのクラスは各FPS（frames per second）の定義です。タイマーが実行間隔を決めるために主にこのenumを使用しています。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "There is an enum definition of FPS in 5 intervals. The `Timer` class `delay` argument is an acceptable `FPS` enum value. For example, specify the `FPS.FPS_60` value to that argument. A timer interval becomes approximately `16.6666667` milliseconds. Similarly, it becomes `33.3333333` milliseconds when you specify the `FPS.FPS_30` value.": "FPSのenumの定義は5間隔（15, 20, 25, 30等）で存在します。`Timer`クラスの`delay`引数は`FPS`のenumの値を受け付けることができます。例えば`FPS.FPS_60`の値を`delay`引数に指定した場合、タイマーの間隔は約`16.6666667`ミリ秒ごととなります。同じように`FPS.FPS_30`を指定すると`33.3333333`ミリ秒ごとの間隔となります。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle_1}\ntimer_1: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_10, options=options)\ntimer_1.start()\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\noptions = {"rectangle": rectangle_2}\ntimer_2: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_30, options=options)\ntimer_2.start()\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=250, y=50, width=50, height=50)\noptions = {"rectangle": rectangle_3}\ntimer_3: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer_3.start()\n\nap.save_overall_html(dest_dir_path="fps_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\nap.Stage(\n    stage_width=350,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle_1}\ntimer_1: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_10, options=options)\ntimer_1.start()\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\noptions = {"rectangle": rectangle_2}\ntimer_2: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_30, options=options)\ntimer_2.start()\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(x=250, y=50, width=50, height=50)\noptions = {"rectangle": rectangle_3}\ntimer_3: ap.Timer = ap.Timer(handler=on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer_3.start()\n\nap.save_overall_html(dest_dir_path="fps_basic_usage/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Timer class delay setting](timer_delay.md)": "- [Timer クラスの delay 設定](jp_timer_delay.md)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: string_upper.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# String class upper method": "# String クラスの upper メソッド",
    ##################################################
    "This page explains the `String` class `upper` method.": "このページでは`String`クラスの`upper`メソッドについて説明します。",  # noqa
    ##################################################
    "## What method is this?": "## メソッド概要",
    ##################################################
    "The `upper` method returns a copied uppercase string.": "`upper`メソッドは大文字に変換された文字列のコピーを返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `upper` method requires no arguments and it returns a `ap.String` type value.": "`upper`メソッドは引数の指定を必要とせず、且つ`ap.String`型の値を返却します。",  # noqa
    ##################################################
    '```py\n# runnable\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nstring: ap.String = ap.String("aBc1_")\nstring = string.upper()\nap.assert_equal(string, "ABC1_")\n\nap.save_overall_html(dest_dir_path="string_upper_basic_usage/")\n```': '```py\n# runnable\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nstring: ap.String = ap.String("aBc1_")\nstring = string.upper()\nap.assert_equal(string, "ABC1_")\n\nap.save_overall_html(dest_dir_path="string_upper_basic_usage/")\n```',  # noqa
    ##################################################
    "## upper method API": "## upper メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a copied upper case string.<hr>": "大文字に変換された文字列のコピーを返却します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `string`: String": "- `string`: String",
    ##################################################
    "  - A copied upper case string.": "  - 大文字変換された文字列のコピー。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> string: ap.String = ap.String("Hello")\n>>> string = string.upper()\n>>> string\nString("HELLO")\n```': '```py\n>>> import apysc as ap\n>>> string: ap.String = ap.String("Hello")\n>>> string = string.upper()\n>>> string\nString("HELLO")\n```',  # noqa
}



"""The class implementation for the `phone_enabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPhoneEnabledIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `phone_enabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M17.38 10.79l-2.2-2.2c-.28-.28-.36-.67-.25-1.02.37-1.12.57-2.32.57-3.57 0-.55.45-1 1-1H20c.55 0 1 .45 1 1 0 9.39-7.61 17-17 17-.55 0-1-.45-1-1v-3.49c0-.55.45-1 1-1 1.24 0 2.45-.2 3.57-.57.35-.12.75-.03 1.02.24l2.2 2.2c2.83-1.45 5.15-3.76 6.59-6.59z"/></svg>'  # noqa



"""The class implementation for the outlined `add_alert` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAddAlertOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `add_alert` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M10.01 21.01c0 1.1.89 1.99 1.99 1.99s1.99-.89 1.99-1.99h-3.98zM12 6c2.76 0 5 2.24 5 5v7H7v-7c0-2.76 2.24-5 5-5zm0-4.5c-.83 0-1.5.67-1.5 1.5v1.17C7.36 4.85 5 7.65 5 11v6l-2 2v1h18v-1l-2-2v-6c0-3.35-2.36-6.15-5.5-6.83V3c0-.83-.67-1.5-1.5-1.5zM13 8h-2v3H8v2h3v3h2v-3h3v-2h-3z"/></svg>'  # noqa



"""The class implementation for the `add_alert` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAddAlertIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `add_alert` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M10.01 21.01c0 1.1.89 1.99 1.99 1.99s1.99-.89 1.99-1.99h-3.98zm8.87-4.19V11c0-3.25-2.25-5.97-5.29-6.69v-.72C13.59 2.71 12.88 2 12 2s-1.59.71-1.59 1.59v.72C7.37 5.03 5.12 7.75 5.12 11v5.82L3 18.94V20h18v-1.06l-2.12-2.12zM16 13.01h-3v3h-2v-3H8V11h3V8h2v3h3v2.01z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: dblclick.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# dblclick interface": "# dblclick インターフェイス",
    ##################################################
    "This page explains the `dblclick` (double-click) interface.": "このページでは`dblclick` (double-click)のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `dblclick` interface binds the double-click event to any `DisplayObject` instance (e.g., `Sprite`\\, `Rectangle`\\, and so on). If you double-click on that instance, this interface calls the registered handler function.": "`dblclick`インターフェイスは`Sprite`や`Rectangle`などの任意の`DisplayObject`のサブクラスのインスタンスへダブルクリック時のイベントを設定します。もし登録したインスタンス上でダブルクリックした場合、登録されているハンドラの関数などが呼び出されます。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "The following page describes the basic mouse event interfaces.": "以下のページでは基本的なマウスイベントの各インターフェイスについて説明しています。",  # noqa
    ##################################################
    "- [Basic mouse event interfaces](mouse_event_basic.md)": "- [基本的なマウスイベントの各インターフェイス](jp_mouse_event_basic.md)",  # noqa
    ##################################################
    "## Basic usage of the dblclick interface": "## dblclick インターフェイスの基本的な使い方",
    ##################################################
    "Each `DisplayObject` instance has the `dblclick` method, and you can bind handlers by that.": "`DisplayObject`のサブクラスの各インスタンスは`dblclick`メソッドを持っており、それを使ってハンドラを登録することができます。",  # noqa
    ##################################################
    "The following example binds the double-click event handler to the rectangle. If you double-click on that instance, the rectangle color changes from cyan to magenta.": "以下のコード例では四角に対してダブルクリック時のハンドラを登録しています。対象の四角のインスタンスをダブルクリックすると四角の色はシアンからマゼンタに変化します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when double-clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#f0a")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.dblclick(on_double_click)\n\nap.save_overall_html(dest_dir_path="./dblclick_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when double-clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#f0a")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.dblclick(on_double_click)\n\nap.save_overall_html(dest_dir_path="./dblclick_basic_usage/")\n```',  # noqa
    ##################################################
    "## Basic usage of the unbind_dblclick interfaces": "## unbind_dblclick の各インターフェイスの基本的な使い方",  # noqa
    ##################################################
    "The `unbind_dblclick` interface can remove the single binding double-click event from a `DisplayObject` instance. The `unbind_dblclick_all` interface removes all double-click events.": "`unbind_dblclick`インターフェイスは`DisplayObject`のサブクラスの任意のインスタンスからダブルクリックイベントのハンドラの設定を取り除きます。また、`unbind_dblclick_all`インターフェイスは対象のインスタンスに設定されているダブルクリックのハンドラ設定を全て取り除きます。",  # noqa
    ##################################################
    "The following example removes the double click event by the `unbind_dblclick` method. If you double-click the rectangle, nothing happens.": "以下のコード例では`unbind_dblclick`メソッドでダブルクリックのイベント設定を取り除いているため、四角をダブルクリックしても何も起きません。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when double-clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#f0a")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.dblclick(on_double_click)\nrectangle.unbind_dblclick(on_double_click)\n\nap.save_overall_html(dest_dir_path="./unbind_dblclick_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when double-clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#f0a")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.dblclick(on_double_click)\nrectangle.unbind_dblclick(on_double_click)\n\nap.save_overall_html(dest_dir_path="./unbind_dblclick_basic_usage/")\n```',  # noqa
    ##################################################
    "## dblclick API": "## dblclick API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add a double-click event listener setting.<hr>": "ダブルクリックイベントのリスナー設定を追加します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Callable that would be called when double-clicking this instance.": "  - このインスタンスをダブルクリックした際に呼ばれる関数やメソッドのハンドラ。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to be passed to a handler.": "  - ハンドラに渡される省略が可能な追加のパラメーターとしての辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `name`: str": "- `name`: str",
    ##################################################
    "  - Handler's name.": "  - ハンドラ名。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.dblclick(on_double_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.dblclick(on_double_click)\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
    ##################################################
    "## unbind_dblclick API": "## unbind_dblclick API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind a specified handler's double click event.<hr>": "指定されたダブルクリックイベントのハンドラー設定を取り除きます。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Unbinding target Callable.": "  - イベント設定を取り除く対象の関数やメソッドなど。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_dblclick(on_double_click)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.dblclick(on_double_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_dblclick(on_double_click)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.dblclick(on_double_click)\n```',  # noqa
    ##################################################
    "## unbind_dblclick_all API": "## unbind_dblclick_all API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind all double click events.<hr>": "全てのダブルクリックイベント設定を取り除きます。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_dblclick_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.dblclick(on_double_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_double_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_dblclick_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.dblclick(on_double_click)\n```',  # noqa
}



"""The class implementation for the outlined `euro_symbol` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialEuroSymbolOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `euro_symbol` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M15 18.5c-2.51 0-4.68-1.42-5.76-3.5H15v-2H8.58c-.05-.33-.08-.66-.08-1s.03-.67.08-1H15V9H9.24C10.32 6.92 12.5 5.5 15 5.5c1.61 0 3.09.59 4.23 1.57L21 5.3C19.41 3.87 17.3 3 15 3c-3.92 0-7.24 2.51-8.48 6H3v2h3.06c-.04.33-.06.66-.06 1s.02.67.06 1H3v2h3.52c1.24 3.49 4.56 6 8.48 6 2.31 0 4.41-.87 6-2.3l-1.78-1.77c-1.13.98-2.6 1.57-4.22 1.57z"/></svg>'  # noqa



"""Class implementation for the truncate-related mix-in.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn


class TruncMixIn:
    @classmethod
    @final
    @add_debug_info_setting(module_name=__name__)
    def trunc(cls, value: Union[Int, Number, String, Boolean]) -> Int:
        """
        Truncate a fraction value from a specified value.

        Parameters
        ----------
        value : Union[Int, Number, String, Boolean]
            A value to truncate a fraction value.
            If a specified value is the `Number`'s, `String`'s, or `Boolean`'s type,
            the return value becomes an `Int`'s value.

        Returns
        -------
        result : Int
            Truncated and converted to `Int`'s value.

        References
        ----------
        - Math max interface
            - https://simon-ritchie.github.io/apysc/en/math_trunc.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> result_int: ap.Int = ap.Math.trunc(value=ap.Int(10))
        >>> result_int
        Int(10)

        >>> result_int = ap.Math.trunc(value=ap.Number(8.5))
        >>> result_int
        Int(8)

        >>> result_int = ap.Math.trunc(value=ap.String("7.6"))
        >>> result_int
        Int(7)

        >>> result_int = ap.Math.trunc(value=ap.Boolean(True))
        >>> result_int
        Int(1)

        >>> result_int = ap.Math.trunc(value=ap.Boolean(False))
        >>> result_int
        Int(0)
        """
        from apysc._expression import expression_data_util

        suffix: str = ""
        if isinstance(value, VariableNameSuffixMixIn):
            suffix = value._variable_name_suffix

        result: Int = Int(0, variable_name_suffix=suffix)
        if isinstance(value, (Int, Number, Boolean)):
            result._value = int(value._value)
        try:
            result._value = int(float(value._value))
        except Exception:
            pass
        expression: str = f"{result.variable_name} = Math.trunc({value.variable_name});"
        expression_data_util.append_js_expression(expression=expression)
        return result



"""The class implementation for the `report_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialReportOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `report_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M11 7h2v2.92l6.91 6.91 1.09-1.1V8.27L15.73 3H8.27L7.18 4.1 11 7.92zm11.27 14.73l-20-20.01L1 2.99l3.64 3.64L3 8.27v7.46L8.27 21h7.46l1.64-1.63L21 23l1.27-1.27zM12 17.3c-.72 0-1.3-.58-1.3-1.3s.58-1.3 1.3-1.3 1.3.58 1.3 1.3-.58 1.3-1.3 1.3z"/></svg>'  # noqa



"""The class implementation for the outlined `published_with_changes` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPublishedWithChangesOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `published_with_changes` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M18.6,19.5H21v2h-6v-6h2v2.73c1.83-1.47,3-3.71,3-6.23c0-4.07-3.06-7.44-7-7.93V2.05c5.05,0.5,9,4.76,9,9.95 C22,14.99,20.68,17.67,18.6,19.5z M4,12c0-2.52,1.17-4.77,3-6.23V8.5h2v-6H3v2h2.4C3.32,6.33,2,9.01,2,12c0,5.19,3.95,9.45,9,9.95 v-2.02C7.06,19.44,4,16.07,4,12z M16.24,8.11l-5.66,5.66l-2.83-2.83l-1.41,1.41l4.24,4.24l7.07-7.07L16.24,8.11z"/></svg>'  # noqa



"""Mix-in class implementation for the `_set_initial_overall_container_coordinates`
method.
"""

from typing_extensions import final

from apysc._display.sprite import Sprite
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class SetInitialOverallContainerCoordinatesMixIn:
    @final
    @arg_validation_decos.is_display_object_container(
        arg_position_index=1,
        optional=False,
    )
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_overall_container_coordinates(
        self,
        *,
        overall_container: Sprite,
        x: Number,
        y: Number,
    ) -> None:
        """
        Set initial overall container coordinates.

        Parameters
        ----------
        overall_container : Sprite
            An overall container.
        x : Number
            X-coordinate.
        y : Number
            Y-coordinate.
        """
        overall_container.x = x
        overall_container.y = y



"""The class implementation for the `favorite_border` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFavoriteBorderIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `favorite_border` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M16.5 3c-1.74 0-3.41.81-4.5 2.09C10.91 3.81 9.24 3 7.5 3 4.42 3 2 5.42 2 8.5c0 3.78 3.4 6.86 8.55 11.54L12 21.35l1.45-1.32C18.6 15.36 22 12.28 22 8.5 22 5.42 19.58 3 16.5 3zm-4.4 15.55l-.1.1-.1-.1C7.14 14.24 4 11.39 4 8.5 4 6.5 5.5 5 7.5 5c1.54 0 3.04.99 3.57 2.36h1.87C13.46 5.99 14.96 5 16.5 5c2 0 3.5 1.5 3.5 3.5 0 2.89-3.14 5.74-7.9 10.05z"/></svg>'  # noqa



"""The class implementation for the `calculate` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCalculateIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `calculate` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M19,3H5C3.9,3,3,3.9,3,5v14c0,1.1,0.9,2,2,2h14c1.1,0,2-0.9,2-2V5C21,3.9,20.1,3,19,3z M13.03,7.06L14.09,6l1.41,1.41 L16.91,6l1.06,1.06l-1.41,1.41l1.41,1.41l-1.06,1.06L15.5,9.54l-1.41,1.41l-1.06-1.06l1.41-1.41L13.03,7.06z M6.25,7.72h5v1.5h-5 V7.72z M11.5,16h-2v2H8v-2H6v-1.5h2v-2h1.5v2h2V16z M18,17.25h-5v-1.5h5V17.25z M18,14.75h-5v-1.5h5V14.75z"/></g></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: mousemove.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# mousemove interface": "# mousemove インターフェイス",
    ##################################################
    "This page explains the `mousemove` interface.": "このページでは`mousemove`のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `mousemove` interface binds the mouse moving event handler to any `DisplayObject` instance. If you move the mouse cursor on that instance, the interface calls the registered handler.": "`mousemove`のインターフェイスは任意の`DisplayObject`のインスタンスへとマウスを動かした時のイベントハンドラの設定を追加します。対象のインスタンス上でマウスカーソルを動かす度に設定されたハンドラが呼ばれます。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "The following page describes the basic mouse event interfaces.": "以下のページでは基本的なマウスイベントの各インターフェイスについて説明しています。",  # noqa
    ##################################################
    "- [Basic mouse event interfaces](mouse_event_basic.md)": "- [基本的なマウスイベントの各インターフェイス](jp_mouse_event_basic.md)",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each `DisplayObject` instance has the `mousemove` method, and you can bind handlers by that.": "`DisplayObject`の各インスタンスは`mousemove`メソッドを持っており、そのインターフェイスを使ってハンドラを設定することができます。",  # noqa
    ##################################################
    "The following example binds the mouse move event handler to the circle. So if you move a mouse cursor on that, the circle follows the cursor position.": "以下のコード例では円に対してマウスを動かしたときのハンドラを設定しています。対象の円の上でマウスカーソルを動かすと、円の位置はカーソルの位置に更新されます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.x = e.stage_x\n    circle.y = e.stage_y\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\ncircle.mousemove(on_mousemove)\n\nap.save_overall_html(dest_dir_path="mousemove_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.x = e.stage_x\n    circle.y = e.stage_y\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\ncircle.mousemove(on_mousemove)\n\nap.save_overall_html(dest_dir_path="mousemove_basic_usage/")\n```',  # noqa
    ##################################################
    "## Unbind interfaces": "## 解除用のインターフェイス",
    ##################################################
    "`unbind_mousemove` interface can remove the binding of the mouse move event from the `DisplayObject`\\.": "`unbind_mousemove`インターフェイスでは`DisplayObject`のインスタンスのマウスを動かした時のイベントのハンドラ設定を解除することができます。",  # noqa
    ##################################################
    "In the following example, the interface removes the mouse move event handler if you click the circle.": "以下のコード例では円をクリックした際にマウスを動かしたときのイベントハンドラの設定を解除するようにしています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.x = e.stage_x\n    circle.y = e.stage_y\n\n\ndef on_click(e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.unbind_mousemove(handler=on_mousemove)\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\ncircle.mousemove(on_mousemove)\ncircle.click(on_click)\n\nap.save_overall_html(dest_dir_path="mousemove_unbind_interface/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.x = e.stage_x\n    circle.y = e.stage_y\n\n\ndef on_click(e: ap.MouseEvent[ap.Circle], options: dict) -> None:\n    """\n    The handler that the circle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    circle: ap.Circle = e.this\n    circle.unbind_mousemove(handler=on_mousemove)\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\ncircle.mousemove(on_mousemove)\ncircle.click(on_click)\n\nap.save_overall_html(dest_dir_path="mousemove_unbind_interface/")\n```',  # noqa
    ##################################################
    "There are also existing the `unbind_mousemove_all` interface. This interface unbinds all the handlers from the target `DisplayObject` instance.": "また、`unbind_mousemove_all`のインターフェイスも存在し、このインターフェイスは`DisplayObject`のインスタンスからマウスを動かしたときのイベントのハンドラ設定を全て解除します。",  # noqa
    ##################################################
    "## mousemove API": "## mousemove API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add mouse move event listener setting.<hr>": "マウスを動かした時のイベント設定を追加します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Callable that would be called when mousemove on this instance.": "  - インスタンス上でマウスを動かした際に呼ばれる関数もしくはメソッド。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to be passed to a handler.": "  - ハンドラに渡される省略が可能な追加のパラメーターとしての辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `name`: str": "- `name`: str",
    ##################################################
    "  - Handler's name.": "  - ハンドラ名。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace("stage_x:", stage_x)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousemove(on_mousemove)\n```': '```py\n>>> import apysc as ap\n>>> def on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace("stage_x:", stage_x)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousemove(on_mousemove)\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
    ##################################################
    "## unbind_mousemove API": "## unbind_mousemove API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind a specified handler's mouse move event.<hr>": "マウスカーソルを動かした際のイベントで指定されたハンドラの設定を解除します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Unbinding target Callable.": "  - イベント設定を取り除く対象の関数やメソッドなど。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace("stage_x:", stage_x)\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.unbind_mousemove(on_mousemove)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousemove(on_mousemove)\n>>> _ = rectangle.click(on_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace("stage_x:", stage_x)\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.unbind_mousemove(on_mousemove)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousemove(on_mousemove)\n>>> _ = rectangle.click(on_click)\n```',  # noqa
    ##################################################
    "## unbind_mousemove_all API": "## unbind_mousemove_all API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind all mouse move events.<hr>": "マウスカーソルを動かしたときのイベントの全てのハンドラ設定を解除します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace("stage_x:", stage_x)\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.unbind_mousemove_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousemove(on_mousemove)\n>>> _ = rectangle.click(on_click)\n```': '```py\n>>> import apysc as ap\n>>> def on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     ap.trace("stage_x:", stage_x)\n>>> def on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.unbind_mousemove_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousemove(on_mousemove)\n>>> _ = rectangle.click(on_click)\n```',  # noqa
}



"""The mix-in implementation for the `foreignObject`'s text property.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class SvgForeignObjectTextMixIn:
    _text: String

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _initialize_text(self, *, text: Union[str, String]) -> None:
        """
        Initialize the `_text` attribute.

        Parameters
        ----------
        text : Union[str, String]
            Text value. An HTML tag is available.
        """
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._type.variable_name_suffix_attr_or_var_mixin import (
            VariableNameSuffixAttrOrVarMixIn,
        )

        suffix: str = ""
        if isinstance(self, VariableNameSuffixAttrOrVarMixIn):
            suffix = self._get_attr_or_variable_name_suffix(value_identifier="text")
        self._text = to_apysc_val_from_builtin.get_copied_string_from_builtin_val(
            string=text,
            variable_name_suffix=suffix,
        )
        self._text = (
            String("<span>", variable_name_suffix=suffix)
            + self._text
            + String("</span>", variable_name_suffix=suffix)
        )



"""The class implementation for the outlined `person_add_disabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPersonAddDisabledOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `person_add_disabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M15 6c1.1 0 2 .9 2 2 0 .99-.73 1.82-1.67 1.97l-2.31-2.31C13.19 6.72 14.01 6 15 6m0-2c-2.21 0-4 1.79-4 4 0 .18.03.35.05.52l3.43 3.43c.17.02.34.05.52.05 2.21 0 4-1.79 4-4s-1.79-4-4-4zm1.69 10.16L22.53 20H23v-2c0-2.14-3.56-3.5-6.31-3.84zm-3.68 1.97L14.88 18H9c.08-.24.88-1.01 2.91-1.57l1.1-.3M1.41 1.71L0 3.12l4 4V10H1v2h3v3h2v-3h2.88l2.51 2.51C9.19 15.11 7 16.3 7 18v2h9.88l4 4 1.41-1.41L1.41 1.71zM6 10v-.88l.88.88H6z"/></svg>'  # noqa



"""The enum definitions for the text-align CSS property.
"""

from enum import Enum


class CssTextAlign(Enum):
    """
    The enum definitions for the text-align CSS property.

    References
    ----------
    - text_align property
        - https://simon-ritchie.github.io/apysc/en/text_align.html
    """

    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"
    JUSTIFY = "justify"



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_line_dot_setting.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase line_dot_setting interface": "# GraphicsBase クラスの line_dot_setting インターフェイス",  # noqa
    ##################################################
    "This page explains the `GraphicsBase` class `line_dot_setting` property interface.": "このページでは`GraphicsBase`クラスの`line_dot_setting`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `line_dot_setting` property interface updates or gets the instance's current line dot setting.": "`line_dot_setting`属性のインターフェイスはインスタンスの線のドット設定の更新もしくは取得を行います。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter or setter interface value becomes (or requires) the `LineDotSetting` instance value.": "getterやsetterのインターフェイスの値は`LineDotSetting`クラスのインスタンスの値となります。",  # noqa
    ##################################################
    "The following example sets the 5-pixel dot to the line:": "以下のコード例では5pxの点線の設定を線に行っています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=100,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=200, y_end=50)\nline.line_dot_setting = ap.LineDotSetting(dot_size=5)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dot_setting_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=100,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=200, y_end=50)\nline.line_dot_setting = ap.LineDotSetting(dot_size=5)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dot_setting_basic_usage/")\n```',  # noqa
    ##################################################
    "## Delete setting": "## 設定の削除",
    ##################################################
    "The `delete_line_dot_setting` interface deletes this line setting.": "`delete_line_dot_setting`インターフェイスはこの線の設定を削除します。",  # noqa
    ##################################################
    "In the following example, if you click the rectangle, the handler deletes the line setting:": "以下の例では四角をクリックするとハンドラ内の処理で線の設定を削除しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler for the click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Rectangle]\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.delete_line_dot_setting()\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#666"),\n    line_color=ap.Color("#fff"),\n    line_thickness=3,\n    line_dot_setting=ap.LineDotSetting(dot_size=3),\n)\nrectangle.click(handler=on_click)\nap.save_overall_html(dest_dir_path="./graphics_base_line_dot_setting_delete_setting/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler for the click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Rectangle]\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.delete_line_dot_setting()\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#666"),\n    line_color=ap.Color("#fff"),\n    line_thickness=3,\n    line_dot_setting=ap.LineDotSetting(dot_size=3),\n)\nrectangle.click(handler=on_click)\nap.save_overall_html(dest_dir_path="./graphics_base_line_dot_setting_delete_setting/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Graphics class line_style interface](graphics_line_style.md)": "- [Graphics クラスの line_style (線のスタイル設定)のインターフェイス](jp_graphics_line_style.md)",  # noqa
    ##################################################
    "## line_dot_setting property API": "## line_dot_setting 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get this instance's line dot setting.<hr>": "このインスタンスの線の点線のスタイル設定を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line_dot_setting`: LineDotSetting or None": "- `line_dot_setting`: LineDotSetting or None",  # noqa
    ##################################################
    "  - Lien dot setting.": "  - 線の点線のスタイル設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_dot_setting = ap.LineDotSetting(dot_size=5)\n>>> line.line_dot_setting.dot_size\nInt(5)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_dot_setting = ap.LineDotSetting(dot_size=5)\n>>> line.line_dot_setting.dot_size\nInt(5)\n```',  # noqa
    ##################################################
    "## delete_line_dot_setting API": "## delete_line_dot_setting のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Delete a current line dot setting.": "現在の点線の設定を削除します。",
}



"""The class implementation for the outlined `cached` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCachedOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `cached` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 8l-4 4h3c0 3.31-2.69 6-6 6-1.01 0-1.97-.25-2.8-.7l-1.46 1.46C8.97 19.54 10.43 20 12 20c4.42 0 8-3.58 8-8h3l-4-4zM6 12c0-3.31 2.69-6 6-6 1.01 0 1.97.25 2.8.7l1.46-1.46C15.03 4.46 13.57 4 12 4c-4.42 0-8 3.58-8 8H1l4 4 4-4H6z"/></svg>'  # noqa



"""Class implementations for time delta-related interfaces.
"""

from typing import TYPE_CHECKING

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._time.days_mixin import DaysMixIn
from apysc._time.total_seconds_mixin import TotalSecondsMixIn
from apysc._type.initial_substitution_exp_mixin import InitialSubstitutionExpMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._time.datetime_ import DateTime


class TimeDelta(
    VariableNameSuffixMixIn,
    InitialSubstitutionExpMixIn,
    TotalSecondsMixIn,
    DaysMixIn,
):
    """
    Class implementations for time delta-related interfaces.
    Subtraction between two `DateTime` instances returns this class's instance.

    Examples
    --------
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)
    >>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 5)
    >>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2
    >>> timedelta_.days
    Int(2)
    >>> timedelta_.total_seconds()
    Number(172800.0)

    References
    ----------
    - TimeDelta class
        - https://simon-ritchie.github.io/apysc/en/timedelta.html
    - TimeDelta class days interface
        - https://simon-ritchie.github.io/apysc/en/timedelta_days.html
    - TimeDelta class total_seconds interface
        - https://simon-ritchie.github.io/apysc/en/timedelta_total_seconds.html
    """

    _left_datetime: "DateTime"
    _right_datetime: "DateTime"

    @arg_validation_decos.is_apysc_datetime(arg_position_index=1)
    @arg_validation_decos.is_apysc_datetime(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        left_datetime: "DateTime",
        right_datetime: "DateTime",
        *,
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        Class implementations for time delta-related interfaces.

        Parameters
        ----------
        left_datetime : DateTime
            Left-side `DateTime` to compare.
        right_datetime : DateTime
            Right-side `DateTime` to compare.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. The `DateTime` class uses this option internally.
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            self._left_datetime = left_datetime
            self._right_datetime = right_datetime
            self._variable_name_suffix = _get_variable_name_suffix_from_datetimes(
                left_datetime=left_datetime,
                right_datetime=right_datetime,
            )
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=var_names.TIME_DELTA,
            )
            self._set_init_total_seconds_value_for_python(
                left_datetime=self._left_datetime,
                right_datetime=self._right_datetime,
            )
            self._set_init_days_value_for_python(
                left_datetime=self._left_datetime,
                right_datetime=self._right_datetime,
            )

            self._append_constructor_expression()

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        from apysc._expression import expression_data_util

        expression: str = self._create_initial_substitution_expression()
        expression = f"var {expression}"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _create_initial_substitution_expression(self) -> str:
        """
        Create an initial value's substitution expression string.

        Returns
        -------
        expression : str
            Created expression string.
        """
        expression: str = (
            f"{self.variable_name} = {self._left_datetime.variable_name}.getTime()"
            f" - {self._right_datetime.variable_name}.getTime();"
        )
        return expression


def _get_variable_name_suffix_from_datetimes(
    *,
    left_datetime: "DateTime",
    right_datetime: "DateTime",
) -> str:
    """
    Get a variable name suffix from specified `DateTime`s instances.

    Parameters
    ----------
    left_datetime : DateTime
        Left-side `DateTime` to compare.
    right_datetime : DateTime
        Right-side `DateTime` to compare.

    Returns
    -------
    variable_name_suffix : str
        A created variable name suffix string.
    """
    if (
        left_datetime._variable_name_suffix == ""
        and right_datetime._variable_name_suffix == ""
    ):
        return ""
    if left_datetime._variable_name_suffix == "":
        return right_datetime._variable_name_suffix
    if right_datetime._variable_name_suffix == "":
        return left_datetime._variable_name_suffix
    variable_name_suffix: str = (
        f"{left_datetime._variable_name_suffix}_"
        f"{right_datetime._variable_name_suffix}"
    )
    return variable_name_suffix



"""This module is for the translation mapping data of the
following document:

Document file: path_move_to.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# PathMoveTo class": "# PathMoveTo クラス",
    ##################################################
    "This page explains the `PathMoveTo` class.": "このページでは`PathMoveTo`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `PathMoveTo` class is the class to set a new position on a path.": "`PathMoveTo`クラスはパスに新しい座標設定を追加するためのクラスです。",  # noqa
    ##################################################
    "This setting only sets coordinates and does not append any vector graphics.": "この設定は座標設定のみを行い、新しいベクターグラフィックスの追加などは行いません。",  # noqa
    ##################################################
    "Mainly, the `Path` class constructor or `draw_path` interfaces use this setting.": "主にこの設定は`Path`クラスのコンストラクタと`draw_path`メソッドのインターフェイスで使用されます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `PathMoveTo` class constructor requires the `x` and `y` arguments.": "`PathMoveTo`クラスのコンストラクタは`x`と`y`の引数指定を必要とします。",  # noqa
    ##################################################
    "The `Path` class constructor or `draw_path` interfaces' `path_data_list` argument requires its instance.": "`Path`クラスのコンストラクタもしくは`draw_path`メソッドのインターフェイスの`path_data_list`引数でそのインスタンスが必要とされます。",  # noqa
    ##################################################
    "The following example sets the x=50 and y=50 coordinates with the `PathMoveTo` instance and then draws the line to x=150 and y=50:": "以下のコード例では`PathMoveTo`のインスタンスを使ってx=50, y=50の座標位置を設定した後にx=150, y=50の位置へ線を描画しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=150, y=50),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_move_to_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id="stage",\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=150, y=50),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_move_to_basic_usage/")\n```',  # noqa
    ##################################################
    "## Relative position setting": "## 相対座標設定",
    ##################################################
    "The constructor's `relative` optional argument changes its behavior.": "コンストラクタの`relative`のオプション引数はその挙動を変更します。",  # noqa
    ##################################################
    "For example, if you set True to its argument, coordinates become relative.": "例として、もしその引数にTrueを指定した場合座標は相対座標として設定されます。",  # noqa
    ##################################################
    "The default setting is False, and it becomes absolute.": "デフォルト値はFalseとなっており、この設定では絶対座標として扱われます。",  # noqa
    ##################################################
    "The following example sets the relative setting and moves to the 50px under position from the current position:": "以下のコード例ではrelativeの設定を行っており、相対的に50px下の位置に移動させています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=150, y=50),\n        ap.PathMoveTo(x=0, y=50, relative=True),\n        ap.PathLineTo(x=0, y=50, relative=True),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_move_to_relative/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\npath: ap.Path = ap.Path(\n    path_data_list=[\n        ap.PathMoveTo(x=50, y=50),\n        ap.PathLineTo(x=150, y=50),\n        ap.PathMoveTo(x=0, y=50, relative=True),\n        ap.PathLineTo(x=0, y=50, relative=True),\n    ],\n    line_color=ap.Color("#0af"),\n    line_thickness=5,\n)\n\nap.save_overall_html(dest_dir_path="path_move_to_relative/")\n```',  # noqa
    ##################################################
    "## PathMoveTo class constructor API": "## PathMoveTo クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Path data class for the SVG `move to` (M).<hr>": "SVGの特定座標への移動（M）のためのパスデータのクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x`: float or Number": "- `x`: float or Number",
    ##################################################
    "  - X-coordinate of the destination point.": "  - 終点のX座標。",
    ##################################################
    "- `y`: float or Number": "- `y`: float or Number",
    ##################################################
    "  - Y-coordinate of the destination point.": "  - 終点のY座標。",
    ##################################################
    "- `relative`: bool or Boolean, default False": "- `relative`: bool or Boolean, default False",  # noqa
    ##################################################
    "  - A boolean value indicates whether the path coordinates are relative or not (absolute).": "  - パスの座標が相対座標として扱うかもしくは絶対座標として扱うかどうかの真偽値。",  # noqa
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=50),\n...         ap.PathLineTo(x=50, y=50),\n...     ]\n... )\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n>>> path: ap.Path = sprite.graphics.draw_path(\n...     path_data_list=[\n...         ap.PathMoveTo(x=0, y=50),\n...         ap.PathLineTo(x=50, y=50),\n...     ]\n... )\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Path class](https://simon-ritchie.github.io/apysc/en/path.html)": "- [Path クラス](https://simon-ritchie.github.io/apysc/jp/jp_path.html)",  # noqa
    ##################################################
    "- [Graphics draw_path interface](https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html)": "- [Graphics クラスの draw_path インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_graphics_draw_path.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: recommended_type_checker_settings.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Recommended type-annotation checker settings": "# 推奨される型アノテーションのチェック設定",
    ##################################################
    "This page explains the type-annotation checker libraries settings, such as the mypy and Pylance (Pyright).": "このページではmypyやPylance（Pyright）の型アノテーションのチェック用のライブラリの設定について説明します。",  # noqa
    ##################################################
    "## Recommended type-annotation checker libraries": "## 推奨される型チェックのライブラリについて",
    ##################################################
    "The apysc library uses the mypy and Pylance (Pyright) checker libraries.": "apyscライブラリではmypyとPylance（Pyright）の型チェックの各ライブラリを使用しています。",  # noqa
    ##################################################
    "So if you want to use the type-annotation checker libraries on the apysc-using project, these two libraries probably become comfortable.": "そのためapyscを使ったプロジェクトではこれらの2つの（いずれかもしくは両方の）ライブラリの使用が相性が良く向いています。",  # noqa
    ##################################################
    "## The ignoring error code of the mypy": "## mypyで無視設定を行っているエラーコード",
    ##################################################
    "The apysc library ignores the mypy misc (miscellaneous) errors with the `--disable-error-code misc` option.": "apyscライブラリでは`--disable-error-code misc`の引数オプションを使用してmypyのmisc（miscellaneous）のエラーコードのエラーを無視しています。",  # noqa
    ##################################################
    "This error code causes many mypy errors on the apysc library, but most are harmless, so the apysc ignores them.": "このエラーコードは多くのエラーをapyscライブラリ上で発生させますが、大半は無害なエラーとなるためapyscライブラリではこれらを無視する形で開発を行っています。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [mypy issue: Decorated property not supported #1362](https://github.com/python/mypy/issues/1362)": "- [mypy issue: Decorated property not supported #1362](https://github.com/python/mypy/issues/1362)",  # noqa
}



"""The class implementation for the `highlight_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHighlightAltIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `highlight_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M17 5h-2V3h2v2zm-2 16h2v-2.59L19.59 21 21 19.59 18.41 17H21v-2h-6v6zm4-12h2V7h-2v2zm0 4h2v-2h-2v2zm-8 8h2v-2h-2v2zM7 5h2V3H7v2zM3 17h2v-2H3v2zm2 4v-2H3c0 1.1.9 2 2 2zM19 3v2h2c0-1.1-.9-2-2-2zm-8 2h2V3h-2v2zM3 9h2V7H3v2zm4 12h2v-2H7v2zm-4-8h2v-2H3v2zm0-8h2V3c-1.1 0-2 .9-2 2z"/></svg>'  # noqa



"""The mix-in class implementation for the axis `line_thickness` value.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class AxisLineThicknessMixIn:
    _line_thickness: Int

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_line_thickness(
        self,
        *,
        line_thickness: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set an initial axis line thickness (line width) setting.

        Parameters
        ----------
        line_thickness : Union[int, Int]
            An axis line thickness (line width) setting.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(line_thickness, Int):
            line_thickness_: Int = Int(
                line_thickness,
                variable_name_suffix=variable_name_suffix,
            )
        else:
            line_thickness_ = line_thickness
        self._line_thickness = line_thickness_



"""This module is for the translation mapping data of the
following document:

Document file: colors.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Colors class": "# Colors クラス",
    ##################################################
    "This page explains the `Colors` class.": "このページでは`Colors`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `Colors` class has `Color` constants, such as the `BLACK_000000`, `GRAY_999999`, `WHITE_FFFFFF`, and `RED_FF0000`.": "`Colors`クラスは`BLACK_000000`や`GRAY_999999`、`WHITE_FFFFFF`、`RED_FF0000`などの`Color`クラスの各定数定義を持ちます。",  # noqa
    ##################################################
    "Each suffix is a hexadecimal color code of its color.": "各サフィックスはその色の16進数のカラーコードになっています。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "These class constants behave the same as a `Color` value.": "これらのクラス定数は`Color`クラスの値と同様に動作します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Colors.GRAY_333333,\n    stage_elem_id="stage",\n)\n\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Colors.CYAN_00FFFF,\n    line_color=ap.Colors.WHITE_FFFFFF,\n    line_thickness=3,\n)\n\nap.save_overall_html(dest_dir_path="./colors_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Colors.GRAY_333333,\n    stage_elem_id="stage",\n)\n\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Colors.CYAN_00FFFF,\n    line_color=ap.Colors.WHITE_FFFFFF,\n    line_thickness=3,\n)\n\nap.save_overall_html(dest_dir_path="./colors_basic_usage/")\n```',  # noqa
    ##################################################
    "## Defined colors": "## 定義されている各色",
    ##################################################
    "<br>\n<details>\n<summary>Display the code block:</summary>": "<br>\n<details>\n<summary>コードブロックを表示:</summary>",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing import List, Tuple\n\nimport apysc as ap\n\nRECT_SIZE: int = 25\nFONT_SIZE: int = 12\nOUTER_MARGIN: int = 20\n_: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=700,\n    stage_height=((len(ap.Colors.get_colors_members()) + 1) // 2) * (RECT_SIZE + 10)\n    + OUTER_MARGIN * 2\n    - 10,\n)\n\ncolor: ap.Color\ncolors_members: List[Tuple[str, ap.Color]] = ap.Colors.get_colors_members()\ncolor_names: List[ap.String] = []\ncolors: List[ap.Color] = []\nfor color_name, color in colors_members:\n    color_names.append(ap.String(color_name))\n    colors.append(color)\nconstant_names_arr: ap.Array[ap.String] = ap.Array(color_names)\ncolors_arr: ap.Array[ap.Color] = ap.Array(colors)\n\ni: ap.Int\nx: ap.Number = ap.Number(0)\ny: ap.Number = ap.Number(0)\nwith ap.ForArrayIndices(constant_names_arr) as i:\n    with ap.If(i % 2 == 0):\n        x.value = OUTER_MARGIN\n    with ap.Else():\n        x.value = 350\n    y.value = (i // 2) * (RECT_SIZE + 10) + 20\n    color = colors_arr[i]\n    constant_name: ap.String = constant_names_arr[i]\n    ap.Rectangle(\n        x=x,\n        y=y,\n        width=RECT_SIZE,\n        height=RECT_SIZE,\n        fill_color=color,\n        line_color=ap.Color("#fff"),\n        line_thickness=1,\n        line_alpha=0.5,\n    )\n    ap.SvgText(\n        text=constant_name,\n        x=x + RECT_SIZE + 10,\n        y=y + RECT_SIZE / 2 + FONT_SIZE / 2 - 2,\n        font_size=FONT_SIZE,\n        fill_color=ap.Color("#ccc"),\n    )\n\nap.save_overall_html(dest_dir_path="./colors_definitions/")\n```': '```py\n# runnable\nfrom typing import List, Tuple\n\nimport apysc as ap\n\nRECT_SIZE: int = 25\nFONT_SIZE: int = 12\nOUTER_MARGIN: int = 20\n_: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=700,\n    stage_height=((len(ap.Colors.get_colors_members()) + 1) // 2) * (RECT_SIZE + 10)\n    + OUTER_MARGIN * 2\n    - 10,\n)\n\ncolor: ap.Color\ncolors_members: List[Tuple[str, ap.Color]] = ap.Colors.get_colors_members()\ncolor_names: List[ap.String] = []\ncolors: List[ap.Color] = []\nfor color_name, color in colors_members:\n    color_names.append(ap.String(color_name))\n    colors.append(color)\nconstant_names_arr: ap.Array[ap.String] = ap.Array(color_names)\ncolors_arr: ap.Array[ap.Color] = ap.Array(colors)\n\ni: ap.Int\nx: ap.Number = ap.Number(0)\ny: ap.Number = ap.Number(0)\nwith ap.ForArrayIndices(constant_names_arr) as i:\n    with ap.If(i % 2 == 0):\n        x.value = OUTER_MARGIN\n    with ap.Else():\n        x.value = 350\n    y.value = (i // 2) * (RECT_SIZE + 10) + 20\n    color = colors_arr[i]\n    constant_name: ap.String = constant_names_arr[i]\n    ap.Rectangle(\n        x=x,\n        y=y,\n        width=RECT_SIZE,\n        height=RECT_SIZE,\n        fill_color=color,\n        line_color=ap.Color("#fff"),\n        line_thickness=1,\n        line_alpha=0.5,\n    )\n    ap.SvgText(\n        text=constant_name,\n        x=x + RECT_SIZE + 10,\n        y=y + RECT_SIZE / 2 + FONT_SIZE / 2 - 2,\n        font_size=FONT_SIZE,\n        fill_color=ap.Color("#ccc"),\n    )\n\nap.save_overall_html(dest_dir_path="./colors_definitions/")\n```',  # noqa
    ##################################################
    "</details>": "</details>",
    ##################################################
    "## References": "## 参考文献・参考資料",
    ##################################################
    "- [Computer Hope, HTML color codes and names](https://www.computerhope.com/htmcolor.html)": "- [Computer Hope, HTML color codes and names](https://www.computerhope.com/jp_htmcolor.html)",  # noqa
}



"""The class implementation for the outlined `mic_none` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMicNoneOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `mic_none` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 14c1.66 0 3-1.34 3-3V5c0-1.66-1.34-3-3-3S9 3.34 9 5v6c0 1.66 1.34 3 3 3zm-1-9c0-.55.45-1 1-1s1 .45 1 1v6c0 .55-.45 1-1 1s-1-.45-1-1V5zm6 6c0 2.76-2.24 5-5 5s-5-2.24-5-5H5c0 3.53 2.61 6.43 6 6.92V21h2v-3.08c3.39-.49 6-3.39 6-6.92h-2z"/></svg>'  # noqa



"""The class implementation for the `fact_check` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFactCheckIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `fact_check` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M20,3H4C2.9,3,2,3.9,2,5v14c0,1.1,0.9,2,2,2h16c1.1,0,2-0.9,2-2V5 C22,3.9,21.1,3,20,3z M10,17H5v-2h5V17z M10,13H5v-2h5V13z M10,9H5V7h5V9z M14.82,15L12,12.16l1.41-1.41l1.41,1.42L17.99,9 l1.42,1.42L14.82,15z" fill-rule="evenodd"/></g></g></svg>'  # noqa



"""The class implementation for the `update` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialUpdateIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `update` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/></g><g><g><g><path d="M21,10.12h-6.78l2.74-2.82c-2.73-2.7-7.15-2.8-9.88-0.1c-2.73,2.71-2.73,7.08,0,9.79s7.15,2.71,9.88,0 C18.32,15.65,19,14.08,19,12.1h2c0,1.98-0.88,4.55-2.64,6.29c-3.51,3.48-9.21,3.48-12.72,0c-3.5-3.47-3.53-9.11-0.02-12.58 s9.14-3.47,12.65,0L21,3V10.12z M12.5,8v4.25l3.5,2.08l-0.72,1.21L11,13V8H12.5z"/></g></g></g></svg>'  # noqa



"""Class implementation for the `Polygon`-related classes'
`_append_constructor_expression` mix-in.
"""

from typing_extensions import final

from apysc._display.points_2d_mixin import Points2DMixIn
from apysc._display.points_var_name_mixin import PointsVarNameMixIn
from apysc._html.debug_mode import add_debug_info_setting


class PolygonAppendConstructorExpressionMixIn(Points2DMixIn, PointsVarNameMixIn):
    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a polygon's constructor expression.
        """
        from apysc._display.append_fill_alpha_attr_expression_mixin import (
            AppendFillAlphaAttrExpressionMixIn,
        )
        from apysc._display.append_fill_color_expression_mixin import (
            AppendFillColorAttrExpressionMixIn,
        )
        from apysc._display.append_line_alpha_attr_expression_mixin import (
            AppendLineAlphaAttrExpressionMixIn,
        )
        from apysc._display.append_line_cap_attr_expression_mixin import (
            AppendLineCapAttrExpressionMixIn,
        )
        from apysc._display.append_line_color_attr_expression_mixin import (
            AppendLineColorAttrExpressionMixIn,
        )
        from apysc._display.append_line_joints_attr_expression_mixin import (
            AppendLineJointsAttrExpressionMixIn,
        )
        from apysc._display.append_line_thickness_attr_expression_mixin import (
            AppendLineThicknessAttrExpressionMixIn,
        )
        from apysc._display.append_x_attr_expression_mixin import (
            AppendXAttrExpressionMixIn,
        )
        from apysc._display.append_y_attr_expression_mixin import (
            AppendYAttrExpressionMixIn,
        )
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util

        INDENT_NUM: int = 2
        stage: Stage = get_stage()
        points_var_name: str
        points_expression: str
        points_var_name, points_expression = self._make_2dim_points_expression()
        expression: str = (
            f"{points_expression}"
            f"\nvar {self.variable_name} = {stage.variable_name}"
            f"\n  .polygon({points_var_name})"
            "\n  .attr({"
        )
        if isinstance(self, AppendFillColorAttrExpressionMixIn):
            expression = self._append_fill_color_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendFillAlphaAttrExpressionMixIn):
            expression = self._append_fill_alpha_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendLineColorAttrExpressionMixIn):
            expression = self._append_line_color_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendLineThicknessAttrExpressionMixIn):
            expression = self._append_line_thickness_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendLineAlphaAttrExpressionMixIn):
            expression = self._append_line_alpha_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendLineCapAttrExpressionMixIn):
            expression = self._append_line_cap_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendLineJointsAttrExpressionMixIn):
            expression = self._append_line_joints_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendXAttrExpressionMixIn):
            expression = self._append_x_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        if isinstance(self, AppendYAttrExpressionMixIn):
            expression = self._append_y_attr_expression(
                expression=expression, indent_num=INDENT_NUM
            )
        expression += "\n  });"
        expression_data_util.append_js_expression(expression=expression)
        self._points_var_name = points_var_name



"""The class implementation for the `bolt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBoltIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `bolt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M11 21h-1l1-7H7.5c-.58 0-.57-.32-.38-.66.19-.34.05-.08.07-.12C8.48 10.94 10.42 7.54 13 3h1l-1 7h3.5c.49 0 .56.33.47.51l-.07.15C12.96 17.55 11 21 11 21z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_line_color.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase line_color interface": "# GraphicsBase クラスの line_color インターフェイス",
    ##################################################
    "This page explains the `GraphicsBase` class `line_color` property interface.": "このページでは`GraphicsBase`クラスの`line_color`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `line_color` property interface updates or gets the instance's line color.": "`line_color`属性のインターフェイスではインスタンスの線の色の更新や取得を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter or setter interface value becomes (or requires) a `Color` value.": "getterとsetterのインターフェイスで扱う値は`Color`型の値となります。",  # noqa
    ##################################################
    "The following example changes the line color (from cyan to magenta and magenta to cyan) when you click the rectangle:": "以下のコード例では四角をクリックした際に線の色をシアンからマゼンタに変更しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    line_color: ap.Color = rectangle.line_color\n    with ap.If(line_color == ap.Color("#00aaff")):\n        rectangle.line_color = ap.Color("#f0a")\n    with ap.Else():\n        rectangle.line_color = ap.Color("#0af")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0"), alpha=0.0)\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_color_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    line_color: ap.Color = rectangle.line_color\n    with ap.If(line_color == ap.Color("#00aaff")):\n        rectangle.line_color = ap.Color("#f0a")\n    with ap.Else():\n        rectangle.line_color = ap.Color("#0af")\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0"), alpha=0.0)\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_color_basic_usage/")\n```',  # noqa
    ##################################################
    "## line_color property API": "## line_color 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get this instance's line color.<hr>": "インスタンスの線の色を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line_color`: Color": "- `line_color`: Color",
    ##################################################
    "  - Current line color (hexadecimal string, e.g., '#00aaff'). If it is not set, it returns the `COLORLESS` constant.": "  - '#00aaff'などの16進数の線の色。もし設定されていない場合は`COLORLESS`の定数が返却されます。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage(\n...     stage_width=150,\n...     stage_height=150,\n...     background_color=ap.Color("#333"),\n...     stage_elem_id="stage",\n... )\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50,\n...     y=50,\n...     width=50,\n...     height=50,\n...     line_color=ap.Color("#0af"),\n...     line_thickness=2,\n... )\n>>> rectangle.line_color\nColor("#00aaff")\n\n>>> rectangle.line_color = ap.Color("#ff00aa")\n>>> rectangle.line_color\nColor("#ff00aa")\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage(\n...     stage_width=150,\n...     stage_height=150,\n...     background_color=ap.Color("#333"),\n...     stage_elem_id="stage",\n... )\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=50,\n...     y=50,\n...     width=50,\n...     height=50,\n...     line_color=ap.Color("#0af"),\n...     line_thickness=2,\n... )\n>>> rectangle.line_color\nColor("#00aaff")\n\n>>> rectangle.line_color = ap.Color("#ff00aa")\n>>> rectangle.line_color\nColor("#ff00aa")\n```',  # noqa
}



"""The class implementation for the `more_time` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMoreTimeIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `more_time` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><polygon points="10,8 10,14 14.7,16.9 15.5,15.7 11.5,13.3 11.5,8"/><path d="M17.92,12c0.05,0.33,0.08,0.66,0.08,1c0,3.9-3.1,7-7,7s-7-3.1-7-7c0-3.9,3.1-7,7-7c0.7,0,1.37,0.1,2,0.29V4.23 C12.36,4.08,11.69,4,11,4c-5,0-9,4-9,9s4,9,9,9s9-4,9-9c0-0.34-0.02-0.67-0.06-1H17.92z"/><polygon points="20,5 20,2 18,2 18,5 15,5 15,7 18,7 18,10 20,10 20,7 23,7 23,5"/></g></g></svg>'  # noqa



"""The enum definitions for the last line's text-align CSS
property (`text-align-last`).
"""

from enum import Enum


class CssTextAlignLast(Enum):
    """
    The enum definitions for the last line's text-align CSS
    property (`text-align-last`).
    """

    AUTO = "auto"
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"
    JUSTIFY = "justify"



"""The mix-in class implementation for `overall_container`-related interfaces.
"""

from typing_extensions import final

from apysc._display.sprite import Sprite
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class OverallContainerMixIn:
    _overall_container: Sprite

    @final
    @arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _initialize_overall_container(self, *, variable_name_suffix: str) -> None:
        """
        Initialize an overall container.

        Parameters
        ----------
        variable_name_suffix : str
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        self._overall_container = Sprite(variable_name_suffix=variable_name_suffix)



"""The class implementation for the `table_view` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialTableViewIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `table_view` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M19,7H9C7.9,7,7,7.9,7,9v10c0,1.1,0.9,2,2,2h10c1.1,0,2-0.9,2-2V9C21,7.9,20.1,7,19,7z M19,9v2H9V9H19z M13,15v-2h2v2H13z M15,17v2h-2v-2H15z M11,15H9v-2h2V15z M17,13h2v2h-2V13z M9,17h2v2H9V17z M17,19v-2h2v2H17z M6,17H5c-1.1,0-2-0.9-2-2V5 c0-1.1,0.9-2,2-2h10c1.1,0,2,0.9,2,2v1h-2V5H5v10h1V17z"/></g></svg>'  # noqa



"""The class implementation for the `file_present` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFilePresentIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `file_present` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M15 2H6c-1.1 0-2 .9-2 2v16c0 1.1.9 2 2 2h12c1.1 0 2-.9 2-2V7l-5-5zM6 20V4h8v4h4v12H6zm10-10v5c0 2.21-1.79 4-4 4s-4-1.79-4-4V8.5c0-1.47 1.26-2.64 2.76-2.49 1.3.13 2.24 1.32 2.24 2.63V15h-2V8.5c0-.28-.22-.5-.5-.5s-.5.22-.5.5V15c0 1.1.9 2 2 2s2-.9 2-2v-5h2z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: timer_start_and_stop.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Timer class start and stop interfaces": "# Timer クラスの start と stop のインターフェイス",
    ##################################################
    "This page explains the interface of the Timer class `start` and `stop` methods.": "このページでTimerクラスの`start`と`stop`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `start` method interface starts a timer. Conversely, the `stop` method interface stops a timer.": "`start`メソッドのインターフェイスはタイマーをスタートさせます。逆に`stop`メソッドのインターフェイスはタイマーを停止させます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each `start` and `stop` method has no arguments. The following example starts the timer when you click the rectangle and stops when the count reaches 100.": "`start`と`stop`の各メソッドは引数を必要としません。以下のコード例では四角をクリックした際にタイマーをスタートさせ、タイマーのカウントが100に達した時点でタイマーを停止させています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    options_: _RectOptions = {"rectangle": e.this}\n    timer: ap.Timer = ap.Timer(\n        handler=on_timer, delay=16, repeat_count=100, options=options_\n    )\n    timer.start()\n    e.this.unbind_click(handler=on_rectangle_click)\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler what a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.x += 1\n    timer: ap.Timer = e.this\n    condition: ap.Boolean = timer.current_count == 100\n    with ap.If(condition):\n        timer.stop()\n\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="timer_start_and_stop_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    options_: _RectOptions = {"rectangle": e.this}\n    timer: ap.Timer = ap.Timer(\n        handler=on_timer, delay=16, repeat_count=100, options=options_\n    )\n    timer.start()\n    e.this.unbind_click(handler=on_rectangle_click)\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler what a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.x += 1\n    timer: ap.Timer = e.this\n    condition: ap.Boolean = timer.current_count == 100\n    with ap.If(condition):\n        timer.stop()\n\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="timer_start_and_stop_basic_usage/")\n```',  # noqa
    ##################################################
    "## start API": "## start API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Start this timer.<hr>": "タイマーを開始します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> _ = ap.Timer(on_timer, delay=33.3, repeat_count=50).start()\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> _ = ap.Timer(on_timer, delay=33.3, repeat_count=50).start()\n```",  # noqa
    ##################################################
    "## stop API": "## stop API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Stop this timer.<hr>": "タイマーを停止します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n...     with ap.If(rectangle.x > 100):\n...         timer: ap.Timer = e.this\n...         timer.stop()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=33.3, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n...     with ap.If(rectangle.x > 100):\n...         timer: ap.Timer = e.this\n...         timer.stop()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=33.3, options=options).start()\n```',  # noqa
}



"""The mix-in class implementation to disable a value modification.
"""


class DisableValueModificationMixIn:
    _disabled_value_modification: bool = False

    def disable_value_modification(self) -> None:
        """
        Disable a value modification. This method is useful when you want to
        prevent the value modification (e.g., the value updating with the
        `value` property).

        Calling this method after the instantiation is recommended.
        """
        self._disabled_value_modification = True

    def raise_if_value_modification_is_disabled(self) -> None:
        """
        Raise an error if the value modification is disabled.

        Raises
        ------
        ValueError
            If the value modification is disabled.
        """
        if self._disabled_value_modification:
            raise ValueError(
                "The value modification of this instance is disabled. "
                "Please created a new value instance or copy this instance."
            )



"""The class implementation for the `queue_play_next` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialQueuePlayNextIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `queue_play_next` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/></g><g><g><g><path d="M21,3H3C1.89,3,1,3.89,1,5v12c0,1.1,0.89,2,2,2h5v2h8v-2h2v-2H3V5h18v8h2V5C23,3.89,22.1,3,21,3z M13,10V7h-2v3H8v2h3v3 h2v-3h3v-2H13z M24,18l-4.5,4.5L18,21l3-3l-3-3l1.5-1.5L24,18z"/></g></g></g></svg>'  # noqa



"""The class implementation for material design's color scheme samples.
"""

from apysc._material_design.color.material_color_scheme import MaterialColorScheme


class MaterialColorSchemeSamples:
    """
    The class for material design's color scheme samples.
    """

    @classmethod
    def create_light_color_schema_sample_brown_1(cls) -> MaterialColorScheme:
        """
        Create a light color scheme sample setting.

        Returns
        -------
        color_scheme : MaterialColorScheme
            A created color scheme sample setting.

        Notes
        -----
        This setting is created using the material-theme-builder - Apache-2.0 license.

        References
        ----------
        - material-theme-builder
            - https://material-foundation.github.io/material-theme-builder/
        - material-theme-builder GitHub
            - https://github.com/material-foundation/material-theme-builder
        - material-theme-builder license
            - https://github.com/material-foundation/material-theme-builder?tab=Apache-2.0-1-ov-file#readme  # noqa
        """
        from apysc._color.color import Color

        return MaterialColorScheme(
            primary=Color("#8F4C38"),
            on_primary=Color("#FFFFFF"),
            secondary=Color("#77574E"),
            on_secondary=Color("#FFFFFF"),
            error=Color("#BA1A1A"),
            on_error=Color("#FFFFFF"),
            surface=Color("#FFF8F6"),
            on_surface=Color("#231917"),
            primary_container=Color("#FFDBD1"),
            on_primary_container=Color("#3A0B01"),
            secondary_container=Color("#FFDBD1"),
            on_secondary_container=Color("#2C150F"),
            tertiary=Color("#6C5D2F"),
            on_tertiary=Color("#FFFFFF"),
            tertiary_container=Color("#F5E1A7"),
            on_tertiary_container=Color("#231B00"),
            error_container=Color("#FFDAD6"),
            on_error_container=Color("#410002"),
            outline=Color("#85736E"),
            outline_variant=Color("#D8C2BC"),
        )

    @classmethod
    def create_dark_color_scheme_sample_brown_1(cls) -> MaterialColorScheme:
        """
        Create a dark color scheme sample setting.

        Returns
        -------
        color_scheme : MaterialColorScheme
            A created color scheme sample setting.

        Notes
        -----
        This setting is created using the material-theme-builder - Apache-2.0 license.

        References
        ----------
        - material-theme-builder
            - https://material-foundation.github.io/material-theme-builder/
        - material-theme-builder GitHub
            - https://github.com/material-foundation/material-theme-builder
        - material-theme-builder license
            - https://github.com/material-foundation/material-theme-builder?tab=Apache-2.0-1-ov-file#readme  # noqa
        """
        from apysc._color.color import Color

        return MaterialColorScheme(
            primary=Color("#FFB5A0"),
            on_primary=Color("#561F0F"),
            secondary=Color("#E7BDB2"),
            on_secondary=Color("#442A22"),
            error=Color("#FFB4AB"),
            on_error=Color("#690005"),
            surface=Color("#1A110F"),
            on_surface=Color("#F1DFDA"),
            primary_container=Color("#723523"),
            on_primary_container=Color("#FFDBD1"),
            secondary_container=Color("#5D4037"),
            on_secondary_container=Color("#FFDBD1"),
            tertiary=Color("#D8C58D"),
            on_tertiary=Color("#3B2F05"),
            tertiary_container=Color("#534619"),
            on_tertiary_container=Color("#F5E1A7"),
            error_container=Color("#93000A"),
            on_error_container=Color("#FFDAD6"),
            outline=Color("#A08C87"),
            outline_variant=Color("#53433F"),
        )



"""The mix-in class implementation for the `y_axis_column_name` value.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class YAxisColumnNameMixIn:
    _y_axis_column_name: String

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_y_axis_column_name(
        self,
        *,
        y_axis_column_name: Union[str, String],
    ) -> None:
        """
        Set an initial y-axis column name.

        Parameters
        ----------
        y_axis_column_name : Union[str, String]
            Y-axis column name.
        """
        if isinstance(y_axis_column_name, String):
            self._y_axis_column_name = y_axis_column_name
        else:
            self._y_axis_column_name = String(y_axis_column_name)



"""The class implementation for the `text_rotation_angleup` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialTextRotationAngleupIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `text_rotation_angleup` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M4.49 4.21L3.43 5.27 7.85 16.4l1.48-1.48-.92-2.19 3.54-3.54 2.19.92 1.48-1.48L4.49 4.21zm3.09 6.8L5.36 6.14l4.87 2.23-2.65 2.64zm12.99-1.68h-4.24l1.41 1.41-8.84 8.84L10.32 21l8.84-8.84 1.41 1.41V9.33z"/></svg>'  # noqa
